@@ -316,5 +316,11 @@ mix_code_file_set_address (mix_code_file_t *file, mix_address_t address)
   }
   return TRUE;
 }
-
   
+/* get details about the source file */
+const gchar *
+mix_code_file_get_source_path (const mix_code_file_t *file)
+{
+  g_return_val_if_fail (file != NULL, NULL);
+  return file->source_path;
+}
