@@ -21,21 +21,6 @@
 
 #include "mix_ins.h"
 
-/* -------------------------------------> MOVE to VM
-static mix_time_t interlock_time_ = 0;
-static const mix_time_t exec_times_[] = {
-  1, 2, 2, 10, 12, 10, 2, 1, 
-  2, 2, 2, 2 ,  2,  2, 2, 2,
-  2, 2, 2, 2 ,  2,  2, 2, 2,
-  2, 2, 2, 2 ,  2,  2, 2, 2,
-  2, 2, 1, 1,   1,  1, 1, 1,
-  1, 1, 1, 1,   1,  1, 1, 1,
-  1, 1, 1, 1,   1,  1, 1, 1,
-  2, 2, 2, 2 ,  2,  2, 2, 2
-};
-------------------------------------------------------*/
-  
-
 
 struct mix_ins_desc_ 
 {
@@ -246,6 +231,32 @@ mix_ins_to_string(const mix_ins_t *ins)
   return result;
 }
 
+extern void
+mix_ins_to_string_in_buffer (const mix_ins_t *ins, gchar *buf, guint len)
+{
+  gboolean needs_f;
+  mix_ins_id_t id;
+  
+  g_return_if_fail (ins != NULL);
+  g_return_if_fail (buf != NULL);
+  id = mix_ins_id_from_ins(*ins);
+  needs_f = ins->fspec != id_to_desc_[id].def_fspec;
+  if ( needs_f ) 
+    g_snprintf (buf, len, "%s\t%s%d,%d(%d:%d)",  
+		mix_get_string_from_id(id),
+		mix_short_is_negative(ins->address)?"-":"",
+		mix_short_magnitude(ins->address), ins->index, 
+		mix_fspec_left(ins->fspec), 
+		mix_fspec_right(ins->fspec));
+  else
+    g_snprintf(buf, len, "%s\t%s%d,%d",
+	       mix_get_string_from_id(id),
+	       mix_short_is_negative(ins->address)?"-":"",
+	       mix_short_magnitude(ins->address), ins->index);
+}
+
+  
+
 void
 mix_ins_print(const mix_ins_t *ins)
 {
