@@ -116,12 +116,6 @@ mix_parser_new (const gchar *in_file)
   
   if ( f == NULL ) return NULL;
   result = g_new (mix_parser_t, 1);
-  if ( result == NULL ) 
-    {
-      g_warning (_("No system resources"));
-      mix_file_delete (f);
-      return NULL;
-    }
   result->symbol_table = mix_symbol_table_new ();
   result->ls_table = mix_symbol_table_new ();
   result->cur_ls = 0;
@@ -496,10 +490,6 @@ mix_parser_add_raw (mix_parser_t *parser, mix_word_t word, guint lineno)
   if ( parser->status == MIX_PERR_NOCOMP || parser->status == MIX_PERR_OK )
     {
       ins_node_ *node = g_new (ins_node_, 1);
-      if ( node == NULL ) {
-	g_warning (_("Unable to allocate memory"));
-	return;
-      }
       node->ins = word;
       node->lineno = lineno;
       g_tree_insert (parser->ins_table, (gpointer)((guint)parser->loc_count),
