@@ -47,6 +47,16 @@ vm_reset_ (mix_vm_t *vm)
       g_tree_destroy (vm->line_table);
       vm->line_table = NULL;
     }
+  if (vm->address_table != NULL)
+    {
+      g_tree_destroy (vm->address_table);
+      vm->address_table = NULL;
+    }
+  if (vm->src_file != NULL)
+    {
+      mix_src_file_delete (vm->src_file);
+      vm->src_file = NULL;
+    }
   bp_clear_all_ (vm);
 }
 
@@ -58,26 +68,32 @@ mix_vm_new (void)
   int i;
   
   mix_vm_t *vm = g_new (struct mix_vm_t,1);
-  g_return_val_if_fail (vm != NULL, NULL);
   vm->line_table = NULL;
+  vm->address_table = NULL;
   vm->symbol_table = NULL;
   
   for (i = 0; i < BD_NO_; ++i)
     vm->devices[i] = NULL;
-    
+  
+  vm->clock = mix_vm_clock_new ();
+  
   vm_reset_ (vm);
   return vm;
 }
 
+
 void
 mix_vm_delete (mix_vm_t * vm)
 {
   int i;
   
   if (vm->line_table != NULL) g_tree_destroy (vm->line_table);
+  if (vm->address_table != NULL) g_tree_destroy (vm->address_table);
   if (vm->symbol_table != NULL) mix_symbol_table_delete (vm->symbol_table);
+  if (vm->src_file != NULL) mix_src_file_delete (vm->src_file);
   for (i = 0; i < BD_NO_; ++i)
     mix_device_delete (vm->devices[i]);
+  mix_vm_clock_delete (vm->clock);
   g_free (vm);
 }
 
@@ -225,9 +241,9 @@ mix_vm_exec_ins (mix_vm_t *vm, const mix_ins_t *ins)
   return (*ins_handlers_[ins->opcode]) (vm,ins);
 }
 
-/* comparison function for the line table tree */
+/* comparison function for the line and address tables tree */
 static gint
-cmp_lineno_ (gconstpointer a, gconstpointer b)
+cmp_uint_ (gconstpointer a, gconstpointer b)
 {
   return GPOINTER_TO_UINT (a) - GPOINTER_TO_UINT (b);
 }
@@ -237,6 +253,7 @@ mix_vm_load_file (mix_vm_t *vm, const gchar *name)
 {
   mix_code_file_t *file;
   mix_ins_desc_t ins;
+  const gchar *sp;
   
   g_return_val_if_fail (vm != NULL, FALSE);
   file = mix_code_file_new_read (name);
@@ -245,15 +262,26 @@ mix_vm_load_file (mix_vm_t *vm, const gchar *name)
   if ( mix_code_file_is_debug (file) )
     {
       vm->symbol_table = mix_code_file_get_symbol_table (file);
-      vm->line_table = g_tree_new (cmp_lineno_);
+      vm->line_table = g_tree_new (cmp_uint_);
+      vm->address_table = g_tree_new (cmp_uint_);
     }
+
+  sp = mix_code_file_get_source_path (file);
+  if (sp != NULL)
+    vm->src_file = mix_src_file_new_for_read (sp);
+  
   while ( mix_code_file_get_ins (file, &ins) ) 
     {
       set_cell_ (vm, ins.address, ins.ins);
       if ( vm->line_table != NULL )
-	g_tree_insert (vm->line_table, 
-		       GUINT_TO_POINTER (ins.lineno),
-		       GUINT_TO_POINTER ((guint)ins.address));
+	{
+	  g_tree_insert (vm->line_table, 
+			 GUINT_TO_POINTER (ins.lineno),
+			 GUINT_TO_POINTER ((guint)ins.address));
+	  g_tree_insert (vm->address_table, 
+			 GUINT_TO_POINTER ((guint)ins.address),
+			 GUINT_TO_POINTER (ins.lineno));
+	}
     }
   set_loc_ (vm, mix_code_file_get_start_addr (file));
   set_start_ (vm, get_loc_ (vm));
@@ -262,6 +290,13 @@ mix_vm_load_file (mix_vm_t *vm, const gchar *name)
   return TRUE;
 }
 
+const mix_src_file_t *
+mix_vm_get_src_file (const mix_vm_t *vm)
+{
+  g_return_val_if_fail (vm != NULL, NULL);
+  return vm->src_file;
+}
+
 const mix_symbol_table_t *
 mix_vm_get_symbol_table (const mix_vm_t *vm)
 {
@@ -276,6 +311,25 @@ mix_vm_get_prog_count (const mix_vm_t *vm)
   return get_loc_ (vm);
 }
 
+/* Reposition program counter and reset state so that a loaded
+   program can be restarted.
+*/
+void
+mix_vm_reset_program (mix_vm_t *vm)
+{
+  guint k;
+  g_return_if_fail (vm != NULL);
+  reset_loc_ (vm);
+  halt_ (vm, FALSE);
+  set_rA_ (vm, MIX_WORD_ZERO);
+  set_rX_ (vm, MIX_WORD_ZERO);
+  set_rJ_ (vm, MIX_WORD_ZERO);
+  for ( k = 1; k < IREG_NO_+1; ++k )
+    set_rI_ (vm, k, MIX_WORD_ZERO);
+  set_over_ (vm, FALSE);
+  set_cmp_ (vm, mix_EQ);
+}
+
 /* continue execution of instructions in memory */
 int
 mix_vm_run (mix_vm_t *vm)
@@ -283,17 +337,13 @@ mix_vm_run (mix_vm_t *vm)
   mix_ins_t ins;
   g_return_val_if_fail (vm != NULL, MIX_VM_ERROR);
   
-  if ( is_halted_ (vm) )
-    {
-      reset_loc_ (vm);	/* set current location to start address */
-      halt_ (vm, FALSE);
-    }
-  
   while ( !is_halted_ (vm) )
     {
       mix_word_to_ins_uncheck (get_cell_ (vm, get_loc_ (vm)), ins);
       if ( !(*ins_handlers_[ins.opcode]) (vm,&ins) ) 
 	return MIX_VM_ERROR;
+      else
+	update_time_ (vm, &ins);
       if (bp_is_set_ (vm, get_loc_ (vm)))
 	return MIX_VM_BREAK;
       if (get_loc_ (vm) >= MIX_VM_CELL_NO) halt_ (vm, TRUE);
@@ -307,44 +357,29 @@ mix_vm_exec_next (mix_vm_t *vm)
 {
   mix_ins_t ins;
   g_return_val_if_fail (vm != NULL, MIX_VM_ERROR);
-  if ( is_halted_ (vm) ) return MIX_VM_HALT;
+  if (get_loc_ (vm) >= MIX_VM_CELL_NO) halt_ (vm, TRUE);
+  if (is_halted_ (vm)) return MIX_VM_HALT;
   mix_word_to_ins_uncheck (get_cell_ (vm, get_loc_ (vm)), ins);
   if (!(*ins_handlers_[ins.opcode]) (vm, &ins))
     return MIX_VM_ERROR;
-  return MIX_VM_BREAK;
+  else
+    update_time_ (vm, &ins);
+  if (is_halted_ (vm)) return MIX_VM_HALT;
+  return bp_is_set_ (vm, get_loc_ (vm)) ? MIX_VM_BREAK : MIX_VM_OK;
 }
 
 /* Breakpoints */
-typedef struct
-{
-  mix_address_t address;
-  gulong lineno;
-} fline_t;
-
-static int
-find_break_ (gpointer key, gpointer value, gpointer data)
-{
-  fline_t *fl = (fline_t *)data;
-  
-  if ( mix_short_new (GPOINTER_TO_UINT (value)) == fl->address )
-    {
-      fl->lineno = GPOINTER_TO_UINT (key);
-      return TRUE;
-    }
-  return FALSE;
-}
-
 gulong
 mix_vm_get_break_lineno (const mix_vm_t *vm)
 {
-  fline_t fl;
-  
   g_return_val_if_fail (vm != NULL, 0);
-  if (vm->line_table == NULL) return 0;
-  fl.lineno = 0;
-  fl.address = get_loc_ (vm);
-  g_tree_traverse (vm->line_table, find_break_, G_IN_ORDER, (gpointer) &fl);
-  return fl.lineno;
+  if (vm->address_table == NULL)
+    return 0;
+  else
+    {
+      gpointer loc = GUINT_TO_POINTER ((guint)get_loc_ (vm));
+      return GPOINTER_TO_UINT (g_tree_lookup (vm->address_table,loc));
+    }
 }
 
 typedef struct 
@@ -444,3 +479,11 @@ mix_vm_clear_all_breakpoints (mix_vm_t *vm)
   bp_clear_all_ (vm);
 }
 
+/* Get the vm uptime, defined as the time spent executing instructions */
+mix_time_t
+mix_vm_get_uptime (const mix_vm_t *vm)
+{
+  g_return_val_if_fail (vm != NULL, 0);
+  return mix_vm_clock_get_time (get_clock_ (vm));
+}
+
