@@ -26,6 +26,7 @@
 
 #include "mix_symbol_table.h"
 #include "mix_device.h"
+#include "mix_src_file.h"
 #include "mix_vm.h"
 
 /* The mix_vm_t type */
@@ -41,8 +42,11 @@ struct mix_vm_t
   mix_device_t * devices[BD_NO_]; 
   mix_address_t start_addr;	/* start address of loaded file */
   GTree *line_table;		/* source line no -> address  */
+  GTree *address_table;		/* adress -> source line no */
   gint8 bp[MEM_CELLS_NO_/8];	/* each bit signals a break point */
+  mix_vm_clock_t *clock;		/* the vm clock */
   mix_symbol_table_t *symbol_table;
+  mix_src_file_t *src_file;	/* source of last loaded code file */
 };
 
 /* Macros for accessing/modifying the above structure.
@@ -64,6 +68,7 @@ enum { A_ = 0, X_, J_, I1_, I2_, I3_, I4_, I5_, I6_ };
 #define get_cmp_(vm) (vm->cmpflag)
 #define get_over_(vm) (vm->overflow)
 #define get_loc_(vm)  (vm->loc_count)
+#define get_clock_(vm) (vm->clock)
 
 #define set_reg_(vm,r,x)			\
 do {						\
@@ -94,6 +99,7 @@ do {						\
 #define halt_(vm,val) ((vm)->is_halted = (val))
 #define set_start_(vm,val) ((vm)->start_addr = (val))
 #define reset_loc_(vm) set_loc_ (vm, vm->start_addr)
+#define update_time_(vm,ins) mix_vm_clock_add_lapse (get_clock_(vm), ins)
 
 /* Breakpoints handling */
 #define bp_clear_all_(vm)	memset (vm->bp, 0, MEM_CELLS_NO_/8)
