@@ -31,6 +31,7 @@
 
 #include <mixlib/mix_vm.h>
 #include <mixlib/mix_vm_dump.h>
+#include <mixlib/mix_eval.h>
 #include "mixvm_command.h"
 
 /* The names of functions that actually do the manipulation. */
@@ -52,6 +53,7 @@ DEC_FUN (sreg_);
 DEC_FUN (scmp_);
 DEC_FUN (sover_);
 DEC_FUN (smem_);
+DEC_FUN (ssym_);
 DEC_FUN (sbp_);
 DEC_FUN (sbpa_);
 DEC_FUN (cbp_);
@@ -59,6 +61,10 @@ DEC_FUN (cbpa_);
 DEC_FUN (cabp_);
 DEC_FUN (compile_);
 DEC_FUN (edit_);
+DEC_FUN (weval_);
+DEC_FUN (w2d_);
+DEC_FUN (tron_);
+DEC_FUN (troff_);
 DEC_FUN (quit_);
 
 /* A structure which contains information on the commands this program
@@ -94,6 +100,7 @@ COMMAND commands[] = {
   { "sover", cmd_sover_, N_("Set overflow flag value"), "sover T | F" },
   { "smem", cmd_smem_, N_("Set memory contents in given address"), 
     "smem ADDRESS VALUE" },
+  { "ssym", cmd_ssym_, N_("Set a symbol\'s value"), "ssym SYMBOL WEXPR" },
   { "sbp", cmd_sbp_, N_("Set break point at given line"), "sbp LINENO" },
   { "cbp", cmd_cbp_, N_("Clear break point at given line"), "cbp LINENO" },
   { "sbpa", cmd_sbpa_, N_("Set break point at given address"),
@@ -103,6 +110,11 @@ COMMAND commands[] = {
   { "cabp", cmd_cabp_, N_("Clear all breakpoints"), "cabp" },
   { "compile", cmd_compile_, N_("Compile a source file"), "compile FILENAME"},
   { "edit", cmd_edit_, N_("Edit a source file"), "edit FILENAME"},
+  { "weval", cmd_weval_, N_("Evaluate a given W-expres sion"), "weval WEXPR"},
+  { "w2d", cmd_w2d_, N_("Convert a MIX word to its decimal value"),
+    "w2d WORD" },
+  { "tron", cmd_tron_, N_("Turn on instruction tracing."), "tron" },
+  { "troff", cmd_troff_, N_("Turn off instruction tracing."), "troff" },
   { "quit", cmd_quit_, N_("Quit the program"), "quit" },
   { (char *)NULL, (Function *)NULL, (char *)NULL }
 };
@@ -179,10 +191,18 @@ find_command_ (const char *name)
   return ((COMMAND *)NULL);
 }
 
-
+
 /* the virtual machine and dump context */
 static mix_vm_t *vm_ = NULL;
 static mix_dump_context_t *dc_ = NULL;
+/* the w-expression evaluator */
+static mix_eval_t *eval_ = NULL;
+/* trace status */
+static gboolean is_tracing_ = FALSE;
+/* uptime */
+static mix_time_t uptime_ = 0;
+static mix_time_t prog_time_ = 0;
+
 
 static int
 cmd_help_ (char *arg)
@@ -229,8 +249,13 @@ cmd_load_ (char *arg)
 {
   errno = 0;
   if (arg == NULL || *arg == '\0')
-    fputs (_("Missing file name\n"), stderr);
-  else if (!mix_vm_load_file (vm_, arg) )
+    {
+      fputs (_("Missing file name\n"), stderr);
+      return TRUE;
+    }
+  mix_eval_remove_symbols_from_table (eval_,
+				      mix_vm_get_symbol_table (vm_));
+  if (!mix_vm_load_file (vm_, arg) )
     {
       fprintf (stderr, _("Cannot load %s: "), arg);
       if ( errno == 0 )
@@ -239,19 +264,79 @@ cmd_load_ (char *arg)
 	perror (NULL);
       return TRUE + 1;
     }
+  mix_eval_set_symbols_from_table (eval_,
+				   mix_vm_get_symbol_table (vm_));
+
   fprintf (stderr, _("Program loaded. Start address: %d\n"),
 	   mix_vm_get_prog_count (vm_));
   
+  prog_time_ = 0;
+  
   return TRUE;
 }
 
+/* trace current instruction */
+static void
+trace_ (void)
+{
+  enum {BUFFER_LEN = 128};
+  static gchar STRINS[BUFFER_LEN];
+  
+  const mix_src_file_t *file = mix_vm_get_src_file (vm_);
+  const gchar *line = "\n";
+  mix_address_t loc = mix_vm_get_prog_count (vm_);
+  mix_word_t ins = mix_vm_get_addr_contents (vm_, loc);
+  mix_ins_t fins;
+  mix_word_to_ins_uncheck (ins, fins);
+  mix_ins_to_string_in_buffer (&fins, STRINS, BUFFER_LEN);
+  
+  if (file != NULL)
+    {
+      gulong b = mix_vm_get_break_lineno (vm_);
+      if (b > 0) line = mix_src_file_get_line (file, b);
+    }
+  
+  printf ("%d: [%s]\t%s", (gint)loc, STRINS, line);
+}
+
+static int
+run_and_trace_ (void)
+{
+  int k = MIX_VM_OK;
+  if (!is_tracing_)
+    return mix_vm_run (vm_);
+  else while (k == MIX_VM_OK)
+    {
+      trace_ ();
+      k = mix_vm_exec_next (vm_);
+    }
+  return k;
+}
+
+static void
+print_time_ (void)
+{
+  mix_time_t lapse = mix_vm_get_uptime(vm_) - uptime_;
+  uptime_ += lapse;
+  prog_time_ += lapse;
+  printf(_("Elapsed time: %ld /Total program time: %ld (Total uptime: %ld)\n"),
+	 lapse, prog_time_, uptime_);
+}
+
 static int
 cmd_run_ (char *arg)
 {
   if (arg != NULL && *arg != '\0' && cmd_load_ (arg) != TRUE) 
       return TRUE;
   puts (_("Running ..."));
-  switch (mix_vm_run (vm_))
+  
+  if (mix_vm_is_halted (vm_))
+    {
+      mix_vm_reset_program (vm_);
+      prog_time_ = 0;
+    }
+  
+  switch (run_and_trace_ ())
     {
     case MIX_VM_HALT:
       puts (_("... done"));
@@ -274,6 +359,7 @@ cmd_run_ (char *arg)
       g_assert_not_reached ();
       break;
     }
+  print_time_ ();
   return TRUE;
 }
 
@@ -281,6 +367,8 @@ static int
 cmd_next_ (char *arg)
 {
   int ins_no = 1;
+  int k;
+
   if ( strlen (arg) != 0 )
     {
       int k = 0;
@@ -294,9 +382,16 @@ cmd_next_ (char *arg)
       ins_no = atoi (arg);
     }
 
+  if (mix_vm_is_halted (vm_))
+    {
+      mix_vm_reset_program (vm_);
+      prog_time_ = 0;
+    }
+  
   while ( ins_no-- > 0 )
     {
-      int k = mix_vm_exec_next (vm_);
+      if (is_tracing_) trace_ ();
+      k = mix_vm_exec_next (vm_);
       if (k == MIX_VM_HALT)
 	{
 	  fprintf (stderr, _("End of program reached at address %d\n"),
@@ -310,7 +405,7 @@ cmd_next_ (char *arg)
 	  break;
 	}
     }
-
+  print_time_ ();
   return TRUE;
 }
 
@@ -333,7 +428,7 @@ cmd_pc_ (char *arg)
 static int
 cmd_psym_ (char *arg)
 {
-  const mix_symbol_table_t *table = mix_vm_get_symbol_table (vm_);
+  const mix_symbol_table_t *table = mix_eval_symbol_table (eval_);
   if ( table == NULL )
     fputs (_("Symbol table not available\n"), stderr);
   else if (arg != NULL && *arg != '\0')
@@ -341,13 +436,13 @@ cmd_psym_ (char *arg)
       if ( mix_symbol_table_is_defined (table, arg) )
 	{
 	  mix_word_print (mix_symbol_table_value (table, arg), NULL);
-	  puts ("\n");
+	  putc ('\n', stdout);
 	}
       else
 	printf (_("%s: symbol not defined\n"), arg);
     }
   else
-    mix_symbol_table_print (table, MIX_SYM_ROWS, stdout, FALSE);
+    mix_symbol_table_print (table, MIX_SYM_ROWS, stdout, TRUE);
       
   return TRUE;
 }
@@ -615,6 +710,35 @@ cmd_smem_ (char *arg)
   return TRUE;
 }
 
+static int
+cmd_ssym_ (char *arg)
+{
+  if (arg == NULL || strlen(arg) == 0)
+    {
+      fprintf (stderr, _("Missing arguments\n"));
+      return cmd_help_ ("ssym");
+    }
+  else
+    {
+      gchar *a = g_strdup (arg);
+      gchar *s = strtok (a, " \t");
+      gchar *w = strtok (NULL, " \t");
+      if (w != NULL && strtok (NULL, " \t") == NULL)
+	{
+	  cmd_weval_ (w);
+	  if (mix_eval_last_error (eval_) == MIX_EVAL_OK)
+	    mix_eval_set_symbol (eval_, s, mix_eval_value (eval_));
+	}
+      else
+	{
+	  fprintf (stderr, _("Wrong argument number\n"));
+	  cmd_help_ ("ssym");
+	}
+      g_free (a);
+      return TRUE;
+    }
+}
+
 static int
 cmd_sbp_ (char *arg)
 {
@@ -789,10 +913,10 @@ cmd_edit_ (char *arg)
   if (!editor)
     {
       int k;
-      fputs ("Cannot find editor (", stderr);
+      fputs (_("Cannot find editor ("), stderr);
       for (k = 0; k < s; k++)
 	fprintf (stderr, "%s ", envars[k]);
-      fputs ("undefined)\n", stderr);
+      fputs (_("undefined)\n"), stderr);
     }
   else
     {
@@ -804,6 +928,97 @@ cmd_edit_ (char *arg)
   return TRUE;
 }
 
+static int
+cmd_weval_ (char *arg)
+{
+  if ( strlen (arg) == 0 )
+    return cmd_help_ ("weval");
+
+  if (mix_eval_expression_with_loc (eval_, arg,
+				    mix_vm_get_prog_count (vm_)) ==
+      MIX_EVAL_OK)
+    {
+      mix_word_print (mix_eval_value (eval_), NULL);
+      putc ('\n', stdout);
+    }
+  else
+    {
+      gint pos = mix_eval_last_error_pos (eval_);
+      gint k, len = strlen (arg);
+      g_assert(pos > -1 && pos <= len);
+      for (k = 0; k<pos; ++k) fputc (arg[k], stderr);
+      fputc ('\n', stderr);
+      for (k = 0; k<pos; ++k) fputc (' ', stderr);
+      for (k = pos; k < len; ++k) fputc (arg[k], stderr);
+      fprintf (stderr, _("\nEvaluation error: %s\n"),
+	       mix_eval_last_error_string (eval_));
+    }
+        
+  return TRUE;
+}
+
+static int
+cmd_w2d_ (char *arg)
+{
+  if ( strlen (arg) == 0 )
+    return cmd_help_ ("w2d");
+  else 
+    {
+      gchar *cp = g_strdup (arg), *a = cp;
+      mix_byte_t bytes[5] = {0, 0, 0, 0, 0};
+      gchar *b;
+      guint k = 0;
+      gboolean is_n = (a[0] == '-'), success = TRUE;
+      if (a[0] == '+' || a[0] == '-') ++a;
+      b = strtok (a, " \t");
+      while (b != NULL && k < 5)
+	{
+	  if (strlen (b) != 2 || !isdigit(b[0]) || !isdigit(b[1]))
+	    {
+	      fprintf (stderr, _("Incorrect byte specification: %s\n"), b);
+	      success = FALSE;
+	      b = NULL;
+	    }
+	  else
+	    {
+	      bytes[k++] = mix_byte_new (atoi (b));
+	      b = strtok (NULL, " \t");
+	    }
+	}
+      if (success) 
+	{
+	  if (strtok (NULL, "\t") != NULL)
+	    fprintf (stderr, _("The expression %s does not fit in a word\n"),
+		     arg);
+	  else
+	    {
+	      mix_word_t w = mix_bytes_to_word (bytes, k);
+	      fprintf (stdout, "%s%ld\n", is_n? "-":"+",
+		       mix_word_magnitude (w));
+	    }
+	}
+      g_free (cp);
+    }
+  
+  return TRUE;
+}
+
+static int
+cmd_tron_ (char *arg)
+{
+  is_tracing_ = TRUE;
+  printf (_("Instruction tracing has been turned ON.\n"));
+  return TRUE;
+}
+
+static int
+cmd_troff_ (char *arg)
+{
+  is_tracing_ = FALSE;
+  printf (_("Instruction tracing has been turned OFF.\n"));
+  return TRUE;
+}
+
 
 /* external interface */
 void
@@ -816,7 +1031,9 @@ mixvm_cmd_init (char *arg)
   dc_ = mix_dump_context_new (MIX_DUMP_DEF_CHANNEL, 
 			      MIX_SHORT_ZERO, MIX_SHORT_ZERO,
 			      MIX_DUMP_ALL);
-  if ( vm_ == NULL || dc_ == NULL )
+  eval_ = mix_eval_new ();
+  
+  if ( vm_ == NULL || dc_ == NULL || eval_ == NULL)
     g_error (_("Failed initialisation (no memory resources)"));
   if (arg)
     cmd_load_ (arg);
@@ -846,6 +1063,9 @@ mixvm_cmd_exec (char *line)
   if (line[i])
     line[i++] = '\0';
      
+  if (word == NULL || strlen (word) == 0)
+    return TRUE;
+  
   command = find_command_ (word);
      
   if (!command)
